# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tastypie import fields
from tastypie.api import Api
from tastypie.authentication import ApiKeyAuthentication
from tastypie.authorization import DjangoAuthorization
from tastypie.resources import ModelResource, ALL, ALL_WITH_RELATIONS

from common.models import JenkinsBuild, JenkinsJob
from smokeng.models import SmokeImage, SmokeResult


# don't allow deletion of objects with this API
ALLOWED_METHODS = ['get', 'post', 'patch']


class WriteAuthentication(ApiKeyAuthentication):
    '''only require authentication for non-GET operations.'''
    def is_authenticated(self, req, **kwargs):
        if req.method == 'GET':
            return True
        return super(WriteAuthentication, self).is_authenticated(req, **kwargs)


class WriteAuthorization(DjangoAuthorization):
    '''only require authorization for non-GET operations.'''

    def is_authorized(self, req, object=None):
        if req.method == 'GET':
            return True
        return super(WriteAuthorization, self).is_authorized(req, object)


class JenkinsJobResource(ModelResource):
    class Meta:
        queryset = JenkinsJob.objects.filter(publish=True)
        resource_name = 'job'

        filtering = {
            'name': ALL,
        }

        authentication = WriteAuthentication()
        authorization = WriteAuthorization()
        allowed_methods = ALLOWED_METHODS


class JenkinsBuildResource(ModelResource):
    job = fields.ForeignKey(JenkinsJobResource, 'job')

    class Meta:
        queryset = JenkinsBuild.objects.filter(publish=True)
        resource_name = 'build'

        filtering = {
            'job': ALL_WITH_RELATIONS,
            'build_number': ALL,
        }

        authentication = WriteAuthentication()
        authorization = WriteAuthorization()
        allowed_methods = ALLOWED_METHODS


class SmokeImageResource(ModelResource):
    class Meta:
        queryset = SmokeImage.objects.filter(publish=True)
        resource_name = 'image'

        filtering = {
            'build_number': ALL,
            'release': ALL,
            'flavor': ALL,
            'variant': ALL,
            'arch': ALL,
        }

        authentication = WriteAuthentication()
        authorization = WriteAuthorization()
        allowed_methods = ALLOWED_METHODS


class SmokeResultResource(ModelResource):
    image = fields.ForeignKey(SmokeImageResource, 'image')
    jenkins_build = fields.ForeignKey(JenkinsBuildResource, 'jenkins_build')

    class Meta:
        queryset = SmokeResult.objects.filter(publish=True)
        resource_name = 'result'

        filtering = {
            'jenkins_build': ALL_WITH_RELATIONS,
            'image': ALL_WITH_RELATIONS,
            'name': ALL,
        }

        authentication = WriteAuthentication()
        authorization = WriteAuthorization()
        allowed_methods = ALLOWED_METHODS


v1_api = Api(api_name='v1')
v1_api.register(JenkinsJobResource())
v1_api.register(JenkinsBuildResource())
v1_api.register(SmokeImageResource())
v1_api.register(SmokeResultResource())
